require 'covere.scenes.play.Core'
require 'covere.scenes.play.Defend'
require 'covere.scenes.play.Spawner'
require 'covere.scenes.play.SlowMotion'
require 'covere.scenes.play.Info'

local play = {}
play.__index = play

function play:init()
    self:setupThing()
    self:setupListener()

    self:start()
end

function play:setupListener()
    listener.add('keypressed', 'escape', function () self:toMenuScene() end)
end

function play:setupThing()
    local core = Core.new()
    core.onDie = function () self:lostLife() end
    
    local defend = Defend.new()

    local spawner = Spawner.new()

    local slomo = SlowMotion.new()

    local info = Info.new()
end

function play:start()
    local defend = Defend.instance()
    defend:add(Gun)
    defend:add(Cover)
    defend:add(Collector)
    defend:choose(1)

    local spawner = Spawner.instance()
    spawner:wave({
    --tutorial
    --
        {'enemy', Enemy, 3, 30, 40, false, math.pi, math.pi/8},
        6,
        {'enemy', Soldier, 1, 80, 100, false, 0, 0},
        5,
    --
        {'enemy', Enemy, 15, 30, 40, true, math.pi/2, math.pi/2},
        10,
        {'enemy', Soldier, 10, 30, 40, true, math.pi, math.pi/2},
        11,
    --
        {'enemy', Enemy, 100, 30, 40},
        12,
    --
        {'item', AutoStorm, 50, 10},
        {'enemy', Enemy, 20, 30, 40, true, math.pi/4, math.pi/2},
        8,
    --
        {'item', TripleShot, 50, 10},
        {'enemy', Soldier, 15, 30, 40, true, math.pi*3/2, math.pi/2},
        10,
    --
        {'item', AutoStorm, 50, 10},
        {'item', TripleShot, 50, 10},
        {'enemy', Enemy, 60, 30, 40},
        12,
    --
        {'item', CombineBack, 50, 15},
        1,
        {'enemy', Enemy, 20, 30, 40, true, 0, math.pi/2},
        {'enemy', Soldier, 15, 30, 40, true, math.pi, math.pi/2},
        15,
        {'item', Recover, 50, 5},
        2,
    --
        {'item', CombineAhead, 50, 15},
        1,
        {'enemy', Enemy, 20, 30, 40, true, math.pi/4, math.pi/2},
        {'enemy', Soldier, 15, 30, 40, true, math.pi*5/4, math.pi/2},
        15,
        {'item', Recover, 50, 5},
        2,
    --
        {'item', Bomber, 100, 15},
        2,
        {'enemy', Enemy, 50, 60, 70},
        {'enemy', Soldier, 50, 60, 70},
        5,
        {'enemy', Enemy, 1, 1000, 1000},
        10,
    --
    --end of tutorial

    --real game
    --[[
    --]]
    }, function () self:win() end)
end

function play:lostLife()
    if Info.instance():lostLife() then
        local core = Core.new()
        core.onDie = function () self:lostLife() end
    else
        play:lose()
    end
end

function play:win()
    director.changeScene('win', Info.instance().score)
end

function play:lose()
    director.changeScene('lose', Info.instance().score)
end

function play:toMenuScene()
    director.changeScene('menu')
end

director.addScene('play', function () setmetatable({}, play):init() end)